==> botanypots-fabric-1.21.1-21.1.7-sources.jar: net/darkhax/botanypots/common/addons/jei/crop/CropCategory.java <==
package net.darkhax.botanypots.common.addons.jei.crop;

import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import mezz.jei.api.gui.builder.IRecipeSlotBuilder;
import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.gui.drawable.IDrawableStatic;
import mezz.jei.api.gui.ingredient.IRecipeSlotDrawable;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.recipe.category.IRecipeCategory;
import net.darkhax.botanypots.common.addons.jei.BotanyPotsJEIPlugin;
import net.darkhax.botanypots.common.api.data.itemdrops.ItemDropProvider;
import net.darkhax.botanypots.common.api.data.recipes.crop.Crop;
import net.darkhax.botanypots.common.api.data.recipes.soil.Soil;
import net.darkhax.botanypots.common.impl.BotanyPotsContent;
import net.minecraft.class_124;
import net.minecraft.class_1799;
import net.minecraft.class_2561;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_8786;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

public class CropCategory implements IRecipeCategory<CropInfo> {

    private final class_2561 title;
    private final IDrawable icon;
    private final IDrawableStatic background;
    private final IDrawableStatic blankSlot;

    public CropCategory(IGuiHelper guiHelper) {
        this.title = class_2561.method_43471("gui.jei.category.botanypots.crop");
        this.icon = guiHelper.createDrawableItemStack(BotanyPotsContent.TAB_ICON.get());
        this.background = guiHelper.createBlankDrawable(155, 57);
        this.blankSlot = guiHelper.getSlotDrawable();
    }

    @NotNull
    @Override
    public RecipeType<CropInfo> getRecipeType() {
        return BotanyPotsJEIPlugin.CROP;
    }

    @NotNull
    @Override
    public class_2561 getTitle() {
        return this.title;
    }

    @NotNull
    @Override
    public IDrawable getBackground() {
        return this.background;
    }

    @Nullable
    @Override
    public IDrawable getIcon() {
        return this.icon;
    }

    @Override
    public void onDisplayedIngredientsUpdate(@NotNull CropInfo recipe, @NotNull List<IRecipeSlotDrawable> recipeSlots, @NotNull IFocusGroup focuses) {
        if (recipeSlots.size() >= 2) {
            recipe.inventory().clear();
            recipe.inventory().add(recipeSlots.get(1).getDisplayedItemStack().orElse(class_1799.field_8037));
            recipe.inventory().add(recipeSlots.get(0).getDisplayedItemStack().orElse(class_1799.field_8037));
            recipe.inventory().add(class_1799.field_8037);
        }
    }

    @Override
    public void setRecipe(@NotNull IRecipeLayoutBuilder builder, @NotNull CropInfo recipe, @NotNull IFocusGroup focuses) {
        // Seed Slot
        final IRecipeSlotBuilder seedSlot = builder.addSlot(RecipeIngredientRole.INPUT, 31, 11);
        seedSlot.addIngredients(recipe.seed());
        seedSlot.addRichTooltipCallback((view, tooltip) -> {
            final class_8786<Crop> crop = Crop.getCrop(class_310.method_1551().field_1687, recipe.context(), recipe.context().getSeedItem());
            if (crop != null) {
                crop.comp_1933().hoverTooltip(recipe.context().getSeedItem(), recipe.context(), class_310.method_1551().field_1687, tooltip::add);
                if (class_310.method_1551().field_1690.field_1827) {
                    tooltip.add(class_2561.method_43469("tooltip.botanypots.crop_id", recipe.id()).method_27692(class_124.field_1063));
                }
            }
        });

        // Soil Slot
        final IRecipeSlotBuilder soilSlot = builder.addSlot(RecipeIngredientRole.INPUT, 31, 29);
        soilSlot.addIngredients(recipe.soil());
        soilSlot.addRichTooltipCallback((view, tooltip) -> {
            if (class_310.method_1551().field_1687 != null) {
                view.getDisplayedItemStack().ifPresent(stack -> {
                    final class_8786<Soil> soil = Soil.getSoil(class_310.method_1551().field_1687, recipe.context(), recipe.context().getSoilItem());
                    if (soil != null) {
                        soil.comp_1933().hoverTooltip(stack, recipe.context(), class_310.method_1551().field_1687, tooltip::add);
                        if (class_310.method_1551().field_1690.field_1827) {
                            tooltip.add(class_2561.method_43469("tooltip.botanypots.soil_id", soil.comp_1932()).method_27692(class_124.field_1063));
                        }
                    }
                });
            }
        });

        // drops
        final IRecipeSlotBuilder[] dropSlots = new IRecipeSlotBuilder[12];
        for (int nextSlotId = 0; nextSlotId < dropSlots.length; nextSlotId++) {
            dropSlots[nextSlotId] = builder.addSlot(RecipeIngredientRole.OUTPUT, 81 + 18 * (nextSlotId % 4), 1 + 18 * (nextSlotId / 4));
        }
        int nextOutputSlot = 0;
        for (ItemDropProvider dropProvider : recipe.drops()) {
            for (class_1799 stack : dropProvider.getDisplayItems()) {
                if (!stack.method_7960()) {
                    dropSlots[nextOutputSlot].addItemStack(stack);
                    nextOutputSlot++;
                    if (nextOutputSlot >= dropSlots.length) {
                        nextOutputSlot = 0;
                    }
                }
            }
        }
    }

    @Override
    public void draw(@NotNull CropInfo recipe, @NotNull IRecipeSlotsView recipeSlotsView, @NotNull class_332 graphics, double mouseX, double mouseY) {
        this.blankSlot.draw(graphics, 30, 10); // Seed
        this.blankSlot.draw(graphics, 30, 28); // Soil
        // Drops
        for (int nextSlotId = 0; nextSlotId < 12; nextSlotId++) {
            this.blankSlot.draw(graphics, 80 + 18 * (nextSlotId % 4), 18 * (nextSlotId / 4));
        }
    }
}

==> botanypots-fabric-1.21.1-21.1.34-sources.jar: net/darkhax/botanypots/common/addons/jei/crop/CropCategory.java <==
package net.darkhax.botanypots.common.addons.jei.crop;

import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import mezz.jei.api.gui.builder.IRecipeSlotBuilder;
import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.gui.drawable.IDrawableStatic;
import mezz.jei.api.gui.ingredient.IRecipeSlotDrawable;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.recipe.category.IRecipeCategory;
import net.darkhax.botanypots.common.addons.jei.BotanyPotsJEIPlugin;
import net.darkhax.botanypots.common.api.data.itemdrops.ItemDropProvider;
import net.darkhax.botanypots.common.api.data.recipes.crop.Crop;
import net.darkhax.botanypots.common.api.data.recipes.soil.Soil;
import net.darkhax.botanypots.common.impl.BotanyPotsContent;
import net.minecraft.class_124;
import net.minecraft.class_1799;
import net.minecraft.class_2561;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_8786;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

public class CropCategory implements IRecipeCategory<CropInfo> {

    private final class_2561 title;
    private final IDrawable icon;
    private final IDrawableStatic background;
    private final IDrawableStatic blankSlot;

    public CropCategory(IGuiHelper guiHelper) {
        this.title = class_2561.method_43471("gui.jei.category.botanypots.crop");
        this.icon = guiHelper.createDrawableItemStack(BotanyPotsContent.TAB_ICON.get());
        this.background = guiHelper.createBlankDrawable(155, 57);
        this.blankSlot = guiHelper.getSlotDrawable();
    }

    @NotNull
    @Override
    public RecipeType<CropInfo> getRecipeType() {
        return BotanyPotsJEIPlugin.CROP;
    }

    @NotNull
    @Override
    public class_2561 getTitle() {
        return this.title;
    }

    @NotNull
    @Override
    public IDrawable getBackground() {
        return this.background;
    }

    @Nullable
    @Override
    public IDrawable getIcon() {
        return this.icon;
    }

    @Override
    public void onDisplayedIngredientsUpdate(@NotNull CropInfo recipe, @NotNull List<IRecipeSlotDrawable> recipeSlots, @NotNull IFocusGroup focuses) {
        if (recipeSlots.size() >= 2) {
            recipe.inventory().clear();
            recipe.inventory().add(recipeSlots.get(1).getDisplayedItemStack().orElse(class_1799.field_8037));
            recipe.inventory().add(recipeSlots.get(0).getDisplayedItemStack().orElse(class_1799.field_8037));
            recipe.inventory().add(class_1799.field_8037);
        }
    }

    @Override
    public void setRecipe(@NotNull IRecipeLayoutBuilder builder, @NotNull CropInfo recipe, @NotNull IFocusGroup focuses) {
        // Seed Slot
        final IRecipeSlotBuilder seedSlot = builder.addSlot(RecipeIngredientRole.INPUT, 31, 11);
        seedSlot.addIngredients(recipe.seed());
        seedSlot.addRichTooltipCallback((view, tooltip) -> {
            final class_8786<Crop> crop = Crop.getCrop(class_310.method_1551().field_1687, recipe.context(), recipe.context().getSeedItem());
            if (crop != null) {
                crop.comp_1933().hoverTooltip(recipe.context().getSeedItem(), recipe.context(), class_310.method_1551().field_1687, tooltip::add);
                if (class_310.method_1551().field_1690.field_1827) {
                    tooltip.add(class_2561.method_43469("tooltip.botanypots.crop_id", recipe.id().toString()).method_27692(class_124.field_1063));
                }
            }
        });

        // Soil Slot
        final IRecipeSlotBuilder soilSlot = builder.addSlot(RecipeIngredientRole.INPUT, 31, 29);
        soilSlot.addIngredients(recipe.soil());
        soilSlot.addRichTooltipCallback((view, tooltip) -> {
            if (class_310.method_1551().field_1687 != null) {
                view.getDisplayedItemStack().ifPresent(stack -> {
                    final class_8786<Soil> soil = Soil.getSoil(class_310.method_1551().field_1687, recipe.context(), recipe.context().getSoilItem());
                    if (soil != null) {
                        soil.comp_1933().hoverTooltip(stack, recipe.context(), class_310.method_1551().field_1687, tooltip::add);
                        if (class_310.method_1551().field_1690.field_1827) {
                            tooltip.add(class_2561.method_43469("tooltip.botanypots.soil_id", soil.comp_1932().toString()).method_27692(class_124.field_1063));
                        }
                    }
                });
            }
        });

        // drops
        final IRecipeSlotBuilder[] dropSlots = new IRecipeSlotBuilder[12];
        for (int nextSlotId = 0; nextSlotId < dropSlots.length; nextSlotId++) {
            dropSlots[nextSlotId] = builder.addSlot(RecipeIngredientRole.OUTPUT, 81 + 18 * (nextSlotId % 4), 1 + 18 * (nextSlotId / 4));
        }
        int nextOutputSlot = 0;
        for (ItemDropProvider dropProvider : recipe.drops()) {
            for (class_1799 stack : dropProvider.getDisplayItems()) {
                if (!stack.method_7960()) {
                    dropSlots[nextOutputSlot].addItemStack(stack);
                    nextOutputSlot++;
                    if (nextOutputSlot >= dropSlots.length) {
                        nextOutputSlot = 0;
                    }
                }
            }
        }
    }

    @Override
    public void draw(@NotNull CropInfo recipe, @NotNull IRecipeSlotsView recipeSlotsView, @NotNull class_332 graphics, double mouseX, double mouseY) {
        this.blankSlot.draw(graphics, 30, 10); // Seed
        this.blankSlot.draw(graphics, 30, 28); // Soil
        // Drops
        for (int nextSlotId = 0; nextSlotId < 12; nextSlotId++) {
            this.blankSlot.draw(graphics, 80 + 18 * (nextSlotId % 4), 18 * (nextSlotId / 4));
        }
    }
}
